import { Component, OnInit, ViewChild } from '@angular/core';
import { IgxDataChartComponent, IgxLineSeriesComponent, IgxStackedAreaSeriesComponent, IgxStackedFragmentSeriesComponent, IgxStackedLineSeriesComponent } from "igniteui-angular-charts";

@Component({
  selector: 'app-chart-page',
  templateUrl: './chart-page.component.html',
  styleUrls: ['./chart-page.component.scss']
})
export class ChartPageComponent implements OnInit {

  public data1: any[] = [];
  public data2: any[] = [];
  public calloutData: any[] = [];

  public valuePaths: string[] = ["StarWarsATAT", "StarWarsOrderAT", "MarvelTower", "MarvelMansion"];
  public seriesTitles: string[] = ["Star Wars AT-AT", "Star Wars Order AT", "Marvel Tower", "Marvel Mansion"];

  @ViewChild("chart", { static: true})
  public chart : IgxDataChartComponent;

  constructor() { }

  ngOnInit(): void {
    this.data1 = [
      { Scenario: 'Scenario A', Date: "Q1", StarWarsATAT: 5, StarWarsOrderAT: 4, MarvelTower: 3, MarvelMansion: 2 },
      { Scenario: 'Scenario A', Date: "Q2", StarWarsATAT: 4, StarWarsOrderAT: 5, MarvelTower: 4, MarvelMansion: 3 },
      { Scenario: 'Scenario A', Date: "Q3", StarWarsATAT: 3, StarWarsOrderAT: 4, MarvelTower: 5, MarvelMansion: 4 },
      { Scenario: 'Scenario A', Date: "Q4", StarWarsATAT: 4, StarWarsOrderAT: 3, MarvelTower: 4, MarvelMansion: 5 },
    ];

    this.data2 = [
      { Scenario: 'Scenario B', Date: "Q1", StarWarsATAT: 7, StarWarsOrderAT: 3, MarvelTower: 5, MarvelMansion: 4 },
      { Scenario: 'Scenario B', Date: "Q2", StarWarsATAT: 2, StarWarsOrderAT: 10, MarvelTower: 7, MarvelMansion: 9 },
      { Scenario: 'Scenario B', Date: "Q3", StarWarsATAT: 6, StarWarsOrderAT: 2, MarvelTower: 9, MarvelMansion: 5 },
      { Scenario: 'Scenario B', Date: "Q4", StarWarsATAT: 1, StarWarsOrderAT: 6, MarvelTower: 7, MarvelMansion: 3 }
    ];

    let previousItem: any = null;
    let index: number = (this.data1.length - 1) / 2; //Calculates the middle of the category x-axis plot area
    for(let i=0; i<this.data1.length; i++){

      let item1 = this.data1[1];
      let item2 = this.data1[2];

      let value1 = 0;
      let value2 = 0;

      let label = this.seriesTitles[i];

      switch (label) {
        case "Star Wars AT-AT": {
          value1 = item1.StarWarsATAT;
          value2 = item2.StarWarsATAT;
          break;
        }
        case "Star Wars Order AT": {
          value1 = item1.StarWarsOrderAT;
          value2 = item2.StarWarsOrderAT;
          break;
        }
        case "Marvel Tower": {
          value1 = item1.MarvelTower;
          value2 = item2.MarvelTower;
          break;
        }
        case "Marvel Mansion": {
          value1 = item1.MarvelMansion;
          value2 = item2.MarvelMansion;
          break;
        }
      }

      let calloutValue = 0;

      calloutValue = (value1 + value2) / 2; //The midpoint between the two points. Needed because index is fractional. Not needed if it's not fractional

      if(previousItem != null){
        calloutValue = (previousItem.Value) + calloutValue; //Add the previous item's value since it is a stacked series.
      }
      else{
        calloutValue = (calloutValue / 2) - 1; //Place the callout in the middle, rather than at the top, with a little bit of margin.
      }

      let item = { Index: index, Value: calloutValue, Label: this.seriesTitles[i] };
      this.calloutData.push(item);

      previousItem = item;
    }
  }

  private chartPopulated = false;
  public onChartRefreshCompleted(){
    if (!this.chartPopulated) {
      if (this.chart != null && this.chart.actualSeries != null && this.chart.actualSeries.length != 0) {
        this.chartPopulated = true;
        this.populateChart();
      }
    }
  }

  public populateChart(){
    let areaSeries : IgxStackedAreaSeriesComponent = this.chart.actualSeries[0] as IgxStackedAreaSeriesComponent;
    let lineSeries : IgxStackedLineSeriesComponent = this.chart.actualSeries[1] as IgxStackedLineSeriesComponent;

    for(let i=0; i<this.valuePaths.length; i++){
      let areaFragment = new IgxStackedFragmentSeriesComponent();
      areaFragment.valueMemberPath = this.valuePaths[i];
      areaFragment.title = "Scenario A " + this.valuePaths[i];
      areaSeries.series.add(areaFragment);

      let lineFragment = new IgxStackedFragmentSeriesComponent();
      lineFragment.valueMemberPath = this.valuePaths[i];
      lineFragment.title = "Scenario B " + this.valuePaths[i];
      lineSeries.series.add(lineFragment);
    }
  }
}
